import pyguetzli


class Test_pyguetzli_api(object):

    def test_consts(self):
        assert hasattr(pyguetzli, "DEFAULT_JPEG_QUALITY")

    def test_process_jpeg_bytes(self):
        assert hasattr(pyguetzli, "process_jpeg_bytes")

    def test_process_rgb_bytes(self):
        assert hasattr(pyguetzli, "process_rgb_bytes")

    def test_process_pil_image(self):
        assert hasattr(pyguetzli, "process_pil_image")



"""
This modules contain helper function to deal with PIL / Pillow Images.

.. note::
    Please note that the ``[PIL]`` (pillow) extra dependency must be installed
    to allow functions from this module to work.
"""


from . import guetzli


def _to_pil_rgb_image(image):
    """Returns an PIL Image converted to the RGB color space. If the image has
    an alpha channel (transparency), it will be overlaid on a black background.

    :param image: the PIL image to convert

    :returns: The input image if it was already in RGB mode, or a new RGB image
              if converted.

    :raises ImportError: PIL / Pillow cannot be imported.
    """
    if image.mode == "RGB":
        return image

    from PIL import Image

    image.load()
    rgb_image = Image.new("RGB", image.size, (0x00, 0x00, 0x00))
    rgb_image.paste(image, mask=image.split()[3])  # bands: R=0, G=1, B=2, A=3

    return rgb_image


def process_pil_image(image, quality=guetzli.DEFAULT_JPEG_QUALITY):
    """Generates an optimized JPEG from a PIL image. If the image has an alpha
    channel (transparency), it will be overlaid on a black background.

    :param image: the PIL image
    :param quality: the output JPEG quality (default 95)

    :returns: Optimized JPEG bytes
    :rtype: bytes

    :raises ImportError: PIL / Pillow cannot be imported.

    .. code:: python

        import pyguetzli
        from PIL import Image

        image = Image.open("./test/image.jpg")
        optimized_jpeg = pyguetzli.process_pil_image(image)
    """
    image_rgb = _to_pil_rgb_image(image)
    image_rgb_bytes = image_rgb.tobytes()
    return guetzli.process_rgb_bytes(
            image_rgb_bytes,
            *image.size,
            quality=quality
            )



import pytest

from pyguetzli import guetzli


class Test_process_jpeg_bytes(object):

    def setup(self):
        self.jpeg_bytes = open("./test/image.jpg", "rb").read()
        self.misc_bytes = b"\x01\x02\x03\x04"

    def test_jpeg_image(self):
        optimized_jpeg_bytes = guetzli.process_jpeg_bytes(self.jpeg_bytes)
        assert isinstance(optimized_jpeg_bytes, bytes)
        assert optimized_jpeg_bytes.startswith(b"\xFF\xD8\xFF\xE0\x00\x10JFIF")
        assert len(optimized_jpeg_bytes) < len(self.jpeg_bytes)

    def test_quality_param(self):
        optimized_jpeg_bytes_q100 = guetzli.process_jpeg_bytes(
                self.jpeg_bytes, 100)
        optimized_jpeg_bytes_q50 = guetzli.process_jpeg_bytes(
                self.jpeg_bytes, 50)
        assert len(optimized_jpeg_bytes_q50) < len(optimized_jpeg_bytes_q100)

    def test_non_jpeg_image(self):
        with pytest.raises(ValueError):
            guetzli.process_jpeg_bytes(self.misc_bytes)

    def test_truncated_jpeg(self):
        with pytest.raises(ValueError):
            guetzli.process_jpeg_bytes(self.jpeg_bytes[:64])


class Test_process_rgb_bytes(object):

    def setup(self):
        self.rgb_bytes = bytes(bytearray([
            0xFF, 0x00, 0x00,   0x00, 0xFF, 0x00,
            0x00, 0x00, 0xFF,   0xFF, 0xFF, 0xFF,
            ]))

    def test_rgb_data(self):
        optimized_jpeg_bytes = guetzli.process_rgb_bytes(self.rgb_bytes, 2, 2)
        assert isinstance(optimized_jpeg_bytes, bytes)
        assert optimized_jpeg_bytes.startswith(b"\xFF\xD8\xFF\xE0\x00\x10JFIF")

    @pytest.mark.skip(reason="we need a larger image to be able to test this")
    def test_quality_param(self):
        optimized_jpeg_bytes_q100 = guetzli.process_rgb_bytes(
                self.rgb_bytes, 2, 2, 100)
        optimized_jpeg_bytes_q50 = guetzli.process_rgb_bytes(
                self.rgb_bytes, 2, 2, 50)
        assert len(optimized_jpeg_bytes_q50) < len(optimized_jpeg_bytes_q100)

    def test_wrong_width_height(self):
        with pytest.raises(ValueError):
            guetzli.process_rgb_bytes(self.rgb_bytes, 4, 4)



from .guetzli import (
        DEFAULT_JPEG_QUALITY,
        process_jpeg_bytes,
        process_rgb_bytes,
        )

from .pil_image import process_pil_image

__all__ = [
        DEFAULT_JPEG_QUALITY,
        process_jpeg_bytes,
        process_rgb_bytes,
        process_pil_image,
        ]



"""
This module contains functions binded from the Guetzli C++ API.
"""


from ._guetzli import lib, ffi


#: Default quality for outputted JPEGs
DEFAULT_JPEG_QUALITY = 95


def process_jpeg_bytes(bytes_in, quality=DEFAULT_JPEG_QUALITY):
    """Generates an optimized JPEG from JPEG-encoded bytes.

    :param bytes_in: the input image's bytes
    :param quality: the output JPEG quality (default 95)

    :returns: Optimized JPEG bytes
    :rtype: bytes

    :raises ValueError: Guetzli was not able to decode the image (the image is
                        probably corrupted or is not a JPEG)

    .. code:: python

        import pyguetzli

        input_jpeg_bytes = open("./test/image.jpg", "rb").read()
        optimized_jpeg = pyguetzli.process_jpeg_bytes(input_jpeg_bytes)
    """
    bytes_out_p = ffi.new("char**")
    bytes_out_p_gc = ffi.gc(bytes_out_p, lib.guetzli_free_bytes)

    length = lib.guetzli_process_jpeg_bytes(
            bytes_in,
            len(bytes_in),
            bytes_out_p_gc,
            quality
            )

    if length == 0:
        raise ValueError("Invalid JPEG: Guetzli was not able to decode the image")  # noqa

    bytes_out = ffi.cast("char*", bytes_out_p_gc[0])
    return ffi.unpack(bytes_out, length)


def process_rgb_bytes(bytes_in, width, height, quality=DEFAULT_JPEG_QUALITY):
    """Generates an optimized JPEG from RGB bytes.

    :param bytes bytes_in: the input image's bytes
    :param int width: the width of the input image
    :param int height: the height of the input image
    :param int quality: the output JPEG quality (default 95)

    :returns: Optimized JPEG bytes
    :rtype: bytes

    :raises ValueError: the given width and height is not coherent with the
                        ``bytes_in`` length.

    .. code:: python

        import pyguetzli

        # 2x2px RGB image
        #                |    red    |   green   |
        image_pixels  = b"\\xFF\\x00\\x00\\x00\\xFF\\x00"
        image_pixels += b"\\x00\\x00\\xFF\\xFF\\xFF\\xFF"
        #                |   blue    |   white   |

        optimized_jpeg = pyguetzli.process_rgb_bytes(image_pixels, 2, 2)
    """
    if len(bytes_in) != width * height * 3:
        raise ValueError("bytes_in length is not coherent with given width and height")  # noqa

    bytes_out_p = ffi.new("char**")
    bytes_out_p_gc = ffi.gc(bytes_out_p, lib.guetzli_free_bytes)

    length = lib.guetzli_process_rgb_bytes(
            bytes_in,
            width,
            height,
            bytes_out_p_gc,
            quality
            )

    bytes_out = ffi.cast("char*", bytes_out_p_gc[0])
    return ffi.unpack(bytes_out, length)



#!/usr/bin/env python
# encoding: UTF-8

import os
import subprocess

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py


class CustomBuildPy(build_py):

    def run(self):
        subprocess.call("cd guetzli/ && make guetzli_static", shell=True)
        build_py.run(self)


long_description = ""
if os.path.isfile("README.rst"):
    long_description = open("README.rst", "r").read()
elif os.path.isfile("README.md"):
    long_description = open("README.md", "r").read()


setup(
    name="pyguetzli",
    version="1.0.0",
    description="Python bindings for Google's Guetzli, a JPEG encoder that optimises JPEG compression",
    url="https://github.com/wanadev/pyguetzli",
    license="Apache-2.0",

    long_description=long_description,
    keywords="image jpeg optimize guetzli",

    author="Wanadev",
    author_email="contact@wanadev.fr",
    maintainer="Fabien LOISON, Alexis BREUST",

    packages=find_packages(),

    setup_requires=["cffi>=1.0.0"],
    install_requires=["cffi>=1.0.0"],
    extra_require = {
        "PIL": ["pillow"]
    },

    cffi_modules=["pyguetzli/guetzli_build.py:ffibuilder"],

    cmdclass={
        "build_py": CustomBuildPy,
    },
)




from PIL import Image

from pyguetzli import pil_image


class Test_process_pil_image(object):

    def setup(self):
        self.pil_image = Image.open("./test/image.jpg")

    def test_pil_image(self):
        optimized_jpeg_bytes = pil_image.process_pil_image(self.pil_image)
        assert isinstance(optimized_jpeg_bytes, bytes)
        assert optimized_jpeg_bytes.startswith(b"\xFF\xD8\xFF\xE0\x00\x10JFIF")

    def test_quality_param(self):
        optimized_jpeg_bytes_q100 = pil_image.process_pil_image(
                self.pil_image, 100)
        optimized_jpeg_bytes_q50 = pil_image.process_pil_image(
                self.pil_image, 50)
        assert len(optimized_jpeg_bytes_q50) < len(optimized_jpeg_bytes_q100)
